import networkx as nx
import community
import random

def GetEdgesIntoDict(links_file):
    edges = {}
    IN = open(links_file, 'r')
    read_line = IN.readline()
    while(read_line):
        t = read_line.rstrip().split()
        if len(t) > 0:
            if t[0] != t[1]:
                if t[0] not in edges:
                    edges[t[0]] = set([t[1]])
                else:
                    edges[t[0]].add(t[1])
                if t[1] not in edges:
                    edges[t[1]] = set([t[0]])
                else:
                    edges[t[1]].add(t[0])
        read_line = IN.readline()
    IN.close()
    return edges

def FirstPartition(edges, first_part_file, method, name):
    OUT = open(first_part_file, 'w')
    node_count = 0
    for n in edges:
        print [node_count, len(edges)]
        node_count = node_count + 1
        if node_count > 0:
            #the nodes
            index = {}
            reverse_index = {}
            count= 0
            to_add_edges = []
            adj = set([])
            for neighbor in edges[n]:
                index[count] = neighbor
                reverse_index[neighbor] = count
                adj.add(neighbor)
                count = count + 1
            for m in reverse_index:
                for k in edges[m]:
                    if k in reverse_index and reverse_index[k] < reverse_index[m]:
                        to_add_edges.append((reverse_index[m], reverse_index[k]))
            G = nx.Graph()
            G.add_nodes_from([i for i in index])
            G.add_edges_from(to_add_edges)
            if len(to_add_edges) > 0:
                dict_H = community.best_partition(G)
                H = {}
                for node in dict_H:
                    if dict_H[node] not in H:
                        H[dict_H[node]] = set([])
                    H[dict_H[node]].add(node)
                
                for i in H:
                    comm = H[i]
                    if len(comm) > 0:
                        for c in comm:
                            OUT.write(str(index[int(c)]) + ' ')
                        OUT.write(str(n))
                        OUT.write('\n')
                    elif len(comm) > 0:
                        for c in comm:
                            if index[int(c)] in edges[n]:
                                OUT.write(str(index[int(c)]) + ' ')
                        OUT.write(str(n))
                        OUT.write('\n')
    OUT.close()

    
def GetMembership(first_part_file, membership_file, min_comm_size):
    node_membership = {}
 
 
    IN = open(first_part_file, 'rb')
    read_line = IN.readline()
    count = 0
    while(read_line):
        t = read_line.rstrip().split()
        if len(t) >= min_comm_size:
            for mem in t:
                if mem not in node_membership:
                    node_membership[mem] = set([])
                node_membership[mem].add(count)
        count = count + 1
        read_line = IN.readline()
     
    IN.close()
     
    OUT = open(membership_file, 'w')
    for n in node_membership:
        in_comms = node_membership[n]
        OUT.write(str(n)+ ' ')
        for c in in_comms:
            OUT.write(str(c) + ' ')
        OUT.write('\n')
    OUT.close()
 
def Jaccard(set1, set2):
    set1 = set(set1)
    set2 = set(set2)
    return float(len(set1.intersection(set2)))/float(len(set1.union(set2)))
  
def GetCommSimilarities(first_part_file, membership_file, sim_file, sim_threshold, min_comm_size):
    print sim_threshold, min_comm_size, 'simfile'
    IN = open(first_part_file, 'rb')
    line_offset = {}
    offset = 0
    count = 0
    for line in IN:
        line_offset[count] = offset
        count = count + 1
        offset += len(line)
    IN.close()
    num_lines = count - 1
     
    IN = open(membership_file, 'r')
    node_membership = {}
    read_line = IN.readline()
    while(read_line):
        t = read_line.rstrip().split()
        if len(t) > 0:
            node = t[0]
            t.remove(node)
            if node not in node_membership:
                node_membership[node] = set([])
            for c in t:
                node_membership[node].add(c)
        read_line = IN.readline()
     
    IN.close()
    IN = open(first_part_file, 'rb')
    OUT = open(sim_file, 'w')
    curr_count = 0
    total_lines = 0
    while(curr_count < num_lines):
#        print [curr_count, num_lines]
        IN.seek(line_offset[curr_count])
        read_line = IN.readline()
        t = read_line.rstrip().split()
        if len(t) >= min_comm_size:
            adj = set([])
            for mem in t:
                adj = adj.union(set(node_membership[mem]))
            for adj_comm in adj:
                if int(adj_comm) > int(curr_count):
#                    print [k for k in line_offset]
#                    print line_offset[int(adj_comm)]
                    IN.seek(line_offset[int(adj_comm)])
                    read_line = IN.readline()
                    r = read_line.rstrip().split()
                    if len(r) >= min_comm_size:    
                        sim = Jaccard(t, r)
                        if r[len(r) - 1] == t[len(t) - 1]:
                            sim = 0
                        if sim > sim_threshold:
                            total_lines = total_lines + 1
                            OUT.write(str(curr_count) + ' ' + str(adj_comm) + ' ' + str(sim) + ' ' + str(len(set(t).intersection(set(r)))) + ' ' + str(len(t)) + ' ' + str(len(r)))
                            OUT.write('\n')
        curr_count = curr_count + 1
    IN.close()
    OUT.close()
    return total_lines
 
def SecondPartition(edges, sim_threshold, overlap_threshold, presence_threshold, sim_file, first_part_file, results_file, total_lines, min_comm_size, clustering_method):
    return_vals = ModClusteringSingleBig(edges, sim_threshold, overlap_threshold, presence_threshold, sim_file, first_part_file, results_file, total_lines, min_comm_size)        
    return return_vals


def ModClusteringSingleBig(edges, sim_threshold, overlap_threshold, presence_threshold, sim_file, first_part_file, results_file, total_lines, min_comm_size):
    IN = open(sim_file, 'r')
    read_line = IN.readline()
    num_lines = 0
    comm_edges = {}
    to_add_edges = []
    while(read_line):
        t = read_line.rstrip().split()
        num_lines += 1
        if len(t) > 0:
            node1 = t[0]
            node2 = t[1]
            sim = t[2]
            overlap = t[3]
            if float(sim) >= sim_threshold and float(overlap) > overlap_threshold:
                if node1 not in comm_edges:
                    comm_edges[node1] = set([])
                if node2 not in comm_edges:
                    comm_edges[node2] = set([])
                if node2 not in comm_edges[node1]:
                    comm_edges[node1].add(node2)
                    comm_edges[node2].add(node1)
#                    to_add_edges.append((node1, node2))
                    weight = sim
                    to_add_edges.append((node1, node2, {'weight':float(weight)}))

        read_line = IN.readline()
    IN.close()
    G = nx.Graph()
    G.add_nodes_from(range(len(comm_edges)))
    G.add_edges_from(to_add_edges)
    dict_H = community.best_partition(G)
    H1 = {}
    for e in dict_H:
        if dict_H[e] not in H1:
            H1[dict_H[e]] = set([])
        H1[dict_H[e]].add(e)
    H = []
    for e in H1:
        H.append(H1[e])
        
    IN = open(first_part_file, 'rb')
    line_offset = {}
    offset = 0
    count = 0
    for line in IN:
        line_offset[count] = offset
        count = count + 1
        offset += len(line)
    IN.close()
    num_lines = count - 1
    
    IN = open(first_part_file, 'rb')
    OUT = open(results_file, 'w')
    all_comms = {}
    i = 0
    for big_comm in H:
#        print 'new=================='
        comm_members = {}
        for comm in big_comm:
            IN.seek(line_offset[int(comm)])
            read_line = IN.readline()
            t = read_line.rstrip().split()
            #print t
            if len(t) > 0:
                for t1 in t:
                    if t1 not in comm_members:
                        comm_members[t1] = 0
                    comm_members[t1] += 1
        all_comms[i] = set([])
        for t1 in comm_members:
            if comm_members[t1] >= 0:#presence_threshold:
                all_comms[i].add(t1)
#        print comm_members
#        print all_comms[i]
        
        for c in all_comms[i]:
            OUT.write(str(c) + '\t')
        OUT.write('\n')
        i += 1
    IN.close()


    return 1

def GetModComms(G):
    dict_H = community.best_partition(G)
    print dict_H
    H1 = {}
    for e in dict_H:
        if dict_H[e] not in H1:
            H1[dict_H[e]] = set([])
        H1[dict_H[e]].add(e)
    H = []
    for e in H1:
        H.append(H1[e])
    return H

def CleanComms(found_file):
    
    IN = open(found_file, 'r')
    OUT = open(found_file + '_cleaned2', 'w')
    comms = {}
    read_line = IN.readline()
    count = 0
    idx = {}
    while(read_line):
        t = read_line.rstrip().split()
        if len(t) > 0:
            comms[count] = set(t)
            for i in t:
                if i not in idx:
                    idx[i] = set([])

                idx[i].add(count)
            count += 1
        elif len(t) > 0:
            comms[count] = set(t)
            count += 1
        read_line = IN.readline()
    IN.close()
    
    for i in range(count):
        C = comms[i]
        if len(C) > 0:
            poss = set([])
            found = 0
            for n in C:

                poss = poss.union(idx[n])
            for j in poss:
                if j < i:
                    if len(comms[j]) == len(comms[i]) and len(comms[j].difference(comms[i])) == 0:
                        found = 1
            if found != 1:
                for n in C:
                    OUT.write(n + '\t')
                OUT.write('\n')
        else:
            for n in C:
                OUT.write(n + '\t')
            OUT.write('\n')
                
    OUT.close()
    

    
